/* mit.h - This file defines the format for the machine information.
   table (MIT). */
/* Copyright (C) 2017 Robin Miyagi
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef ____MIT_H____
#define ____MIT_H____

/********************************************************************/
/*                                                                  */
/*			   Project Includes                         */
/*                                                                  */
/********************************************************************/
/********************************************************************/
/*                                                                  */
/*			   System Includes                          */
/*                                                                  */
/********************************************************************/
#include <stdint.h>
#include <stdbool.h>
/********************************************************************/
/*                                                                  */
/*			  Macro Definitions                         */
/*                                                                  */
/********************************************************************/

/********************************************************************/
/*                                                                  */
/*			      Data Types                            */
/*                                                                  */
/********************************************************************/
/*
 * Type for the ACPI 3.0 Extended Attributes bitfield.
 */
typedef union acpi_union acpi_T;
union acpi_union
{
  uint32_t i;
  struct
  {
    uint32_t ignore:1;
    uint32_t non_volatile:1;
    uint32_t reserved:30;
  } bits;
};
/*
 * Memory map (MM) type.
 */
typedef struct mm_struct mm_T;
struct mm_struct
{
  uint32_t base_address_low;
  uint32_t base_address_high;
  uint32_t length_low;
  uint32_t length_high;
  uint32_t type;	/* Region type, see macro definitions below. */
  acpi_T eab;
} __attribute__((__packed__));
/*
 * Macro definitions for region type.
 */
#define MM_REGION_TYPE_USABLE 0x1
#define MM_REGION_TYPE_RESERVED 0x2
#define MM_REGION_TYPE_RECLAIMABLE 0x3
#define MM_REGION_TYPE_NVS 0x4
#define MM_REGION_TYPE_BAD 0x5
/*
 * Machine information table (MIT) type. 
 */
typedef struct mit_union mit_T;
struct mit_union
{
  mm_T *memory_map;
  void *mbz;	/* must be zero */
  uint32_t map_count;		/* how many entries */
} __attribute__((__packed__)); 
/********************************************************************/
/*                                                                  */
/*			   Global Variables                         */
/*                                                                  */
/********************************************************************/
/********************************************************************/
/*                                                                  */
/*			    Local Variables                         */
/*                                                                  */
/********************************************************************/
/*
 * Reported size from the int $0x15, %eax=0xe820 BIOS function.
 */
static uint8_t mm_reported_size; /* See macro definitions below. */
#define MM_SIZE_EAB 24		 /* If int $0x15, %eax=0xe820 sets
				    %ecx to this value, then EAB is
				    supported. */
/*
 * Last reported size.
 */
static uint8_t mm_last_size;
/*
 * If the int $0x15, %eax=0xe820 BIOS function fails, then this
 * variable is set to `true'.
 */
static bool mm_e820_error = false;
/*
 * If the version is invalid, then this variable is set to `true'.
 */
static bool mm_version_error = false;
/*
 * The reported version.
 */
static uint32_t mm_reported_version;
/********************************************************************/
/*                                                                  */
/*			 Interface Functions                        */
/*                                                                  */
/********************************************************************/
/********************************************************************/
/*                                                                  */
/*			   Inline Functions                         */
/*                                                                  */
/********************************************************************/
static inline uint32_t get_mm_entry (mm_T *entry, uint32_t previous_return);
/********************************************************************/
/*                                                                  */
/*		    Inline Function Implementation                  */
/*                                                                  */
/********************************************************************/
static inline uint32_t
get_mm_entry (mm_T *entry, uint32_t previous_return)
{
  #define MM_SIGNATURE 0x534d4150
  uint8_t size;
  uint32_t signature;
  uint32_t ret;
  
  asm ("int $0x15"
       : "=a" (signature),
	 "=c" (size),
	 "=b" (ret)
       : "a" ((uint32_t) 0xe820),
	 "b" (previous_return),
	 "D" (entry),
	 "c" ((uint32_t) sizeof (mm_T)),
	 "d" ((uint32_t) MM_SIGNATURE)
       :
       );
  asm goto ("jc %l0"
	    :
	    :
	    :
	    : err
	    );
  if (previous_return == 0)
    {
      mm_reported_size = size;
      mm_reported_version = signature;
    }
  mm_last_size = size;
  if (signature != MM_SIGNATURE)
    mm_version_error = true;
 quit:
  return ret;
 err:
  mm_e820_error = true;
  goto quit;
}
#endif	/* ! ____MIT_H____ */
